@@ -4,7 +4,7 @@ require Exporter;
 @EXPORT_OK = qw/json_to_perl valid_json/;
 use warnings;
 use strict;
-our $VERSION = 0.07;
+our $VERSION = 0.09;
 use XSLoader;
 XSLoader::load 'JSON::Parse', $VERSION;
 
@@ -14,6 +14,8 @@ __END__
 
 =pod
 
+=encoding utf-8
+
 =head1 NAME
 
 JSON::Parse - Convert JSON into a Perl variable
@@ -27,6 +29,26 @@ JSON::Parse - Convert JSON into a Perl variable
 
 Convert JSON (JavaScript Object Notation) into Perl.
 
+=head1 DESCRIPTION
+
+JSON::Parse converts a string containing information in the JSON
+(JavaScript Object Notation) format, as specified in L</RFC 4627>,
+into an equivalent Perl structure. It has a single procedural
+interface, L</json_to_perl>. This procedural interface takes one
+argument, a JSON string, and returns one argument, a Perl
+reference. Its input is a complete JSON structure including all
+opening and closing parentheses.
+
+The procedural interface does not allow the user to specify
+conversions to and from different encodings. If its input is marked as
+Unicode characters, the strings in its output are also marked as
+Unicode characters.
+
+JSON::Parse also provides a fast validation function, L</valid_json>,
+which is considerably faster than L</json_to_perl>.
+
+JSON::Parse is based on C and users require a C compiler to install it.
+
 =head1 FUNCTIONS
 
 =head2 valid_json
@@ -52,40 +74,81 @@ This function converts JSON into a Perl structure.
 =head3 Return value
 
 If the first argument does not contain a valid JSON text, the return
-value is the undefined value.
+value is the undefined value. (Actually, L</json_to_perl> throws a
+fatal error, so the return value is irrelevant.)
 
 If the first argument contains a valid JSON text, the return value is
 either a hash reference or an array reference, depending on whether
-the input JSON text is a serialized object or a serialized array.
+the input JSON text is a serialized object or a serialized array. So,
 
-=head3 Mapping from JSON to Perl
+    my $perl = json_to_perl ('["a", "b", "c"]');
+    print ref $perl, "\n";
+    # Prints "ARRAY".
+
+    my $perl = json_to_perl ('{"a":1, "b":2}');
+    print ref $perl, "\n";
+    # Prints "HASH".
+
+=head1 Mapping from JSON to Perl
 
 JSON elements are mapped to Perl as follows:
 
-=over
+=head2 JSON numbers
 
-=item JSON numbers
+A JSON number is inserted into Perl as a string. No conversion from
+the character string to a numerical value is done. (The conversion is
+not done because Perl will do the conversion into a numerical value
+automatically when the scalar is used in a numerical context.)
 
-JSON numbers are mapped to Perl scalars. The JSON number is inserted
-into Perl as a string. No conversion from the character string to a
-numerical value is done.
+Thus
 
-=item JSON strings
+    my $q = @{json_to_perl ('[0.12345]')}[0];
+
+has the same result as a Perl declaration of the form
+
+    my $q = '0.12345';
+
+Notice that this is inserted as a string. This makes no difference,
+because Perl does the conversion anyway:
+
+    print '0.12345' * 5;
+    # prints 0.61725
+
+See also L</Numbers not checked>.
+
+=head2 JSON strings
 
 JSON strings are mapped to Perl scalars as strings. 
 
-JSON escape characters (see page five of L</RFC 4267>) are mapped to
-the equivalent ASCII character before they are passed to Perl. Unicode
-escape characters of the form \uXXXX (see page three of L</RFC 4267>)
-are mapped to UTF-8 octets. This is done regardless of what input
-encoding might be used.
+JSON escape characters such as \t for the tab character (see section
+2.5 of L</RFC 4627> for a full list) are mapped to the equivalent
+ASCII character before they are passed to Perl. Unicode escape
+characters of the form \uXXXX (see page three of L</RFC 4627>) are
+mapped to UTF-8 octets. This is done regardless of what input encoding
+might be used in the JSON text.
+
+=head3 Handling of Unicode
 
 If the input to L</json_to_perl> is marked as Unicode characters, the
 output strings will be marked as Unicode characters. If the input is
 not marked as Unicode characters, the output strings will not be
-marked as Unicode characters.
+marked as Unicode characters. Thus, 
 
-=item JSON arrays
+    use utf8;
+    my $sasori = '["蠍"]';
+    my $p = json_to_perl ($sasori);
+    print utf8::is_utf8 ($p->[0]);
+    # Prints 1.
+
+but
+
+    no utf8;
+    my $ebi = '["海老"]';
+    my $p = json_to_perl ($ebi);
+    print utf8::is_utf8 ($p->[0]);
+    # Prints nothing.
+
+=head2 JSON arrays
 
 JSON arrays are mapped to Perl array references, with elements in the
 same order as they appear in the JSON.
@@ -98,7 +161,7 @@ has the same result as a Perl declaration of the form
 
     my $p = [ 'monday', 'tuesday', 'wednesday' ];
 
-=item JSON objects
+=head2 JSON objects
 
 JSON objects are mapped to Perl hashes (associative arrays). The
 members of the object are mapped to pairs of key and value in the Perl
@@ -124,46 +187,60 @@ has the same result as a Perl declaration of the form
         friday => 'Gotta get down on Friday',
     };
 
-=item null
+=head2 null
 
-The JSON null literal is mapped to the undefined value.
+The JSON null literal is mapped to the undefined value. See L</False = null = undefined value>.
 
-=item true
+=head2 true
 
 The JSON true literal is mapped to a Perl string with the value 'true'.
 
-=item false
+=head2 false
 
-The JSON false literal is mapped to the undefined value.
-
-=back
+The JSON false literal is mapped to the undefined value. See L</False = null = undefined value>.
 
 =head1 RESTRICTIONS
 
-This module imposes the following restrictions on its input
+This module imposes the following restrictions on its input.
 
 =over
 
 =item JSON only
 
-JSON::Parse will only parse JSON which meets the criteria of L</RFC
-4267>.
+JSON::Parse is a strict parser which only parses JSON which exactly
+meets the criteria of L</RFC 4627>. That means, for example, the
+module does not accept single quotes (') instead of double quotes ("),
+or numbers with leading zeros, like 0123.
 
 =item No incremental parsing
 
-JSON::Parse does not do incremental parsing. JSON::Parse will only parse
-a fully-formed JSON string including opening and closing brackets.
+JSON::Parse does not do incremental parsing. In other words,
+JSON::Parse only parses fully-formed JSON strings which include
+opening and closing brackets.
 
 =item UTF-8 only
 
 JSON::Parse is only designed to parse bytes which are in the UTF-8
 format. This is regardless of whether Perl thinks that the text is in
-UTF-8 format (whether Perl has set its utf8 flag for the string). If
-the input is in a different encoding than UTF-8-encoded Unicode, it is
+UTF-8 format (whether Perl has set its internal C<utf8> flag for the
+string). If input is in a different Unicode encoding than UTF-8, it is
 suggested that the user converts the input before handing it to this
-module. The Perl language already provides very extensive facilities
-for transforming encodings in the L<Encode> module, so there seems
-little point in this module duplicating them.
+module. For example, if the user has a string in the UTF-16 format,
+
+    use Encode 'decode';
+    my $input_utf8 = decode ('UTF-16', $input);
+    my $perl = json_to_perl ($input_utf8);
+
+or, if the user is inputing from a file, use the equivalent pragma:
+
+    open my $input, "<:encoding(UTF-16)", 'some-json-file'; 
+
+This is because the Perl language already provides very extensive
+facilities for transforming encodings, such as the L<Encode> module,
+and there is little point in this module duplicating them.
+
+The module does not attempt to determine the nature of the octet
+stream, as described in part 3 of L</RFC 4627>.
 
 =back
 
@@ -174,7 +251,7 @@ known. These may be resolved in a later version.
 
 =over
 
-=item False == null == undefined value
+=item False = null = undefined value
 
 At the moment, both of "false" and "null" in JSON are mapped to the
 undefined value. "true" is mapped to the string "true".
@@ -197,6 +274,17 @@ is a speculation which has not been tested in practice.
 The line numbers in the error messages are off by one (line 1 is line
 0).
 
+=item Compilation failure on Windows compilers
+
+The C code which JSON::Parse is based on uses some dialect features of
+the GNU version of C, and it will not compile on some Microsoft
+Windows compilers.
+
+=item C compiler required
+
+At the moment there is no "pure Perl" version of the module, so use of
+the module requires a C compiler.
+
 =back
 
 =head1 DIAGNOSTICS
@@ -207,27 +295,25 @@ messages:
 
 =over
 
-=item unknown failure
+=item * unknown failure
 
-=item a callback routine failed
+=item * a callback routine failed
 
-=item out of memory
+=item * out of memory
 
-=item parser failed (this JSON is not grammatically correct)
+=item * parser failed (this JSON is not grammatically correct)
 
-=item lexer failed (there are stray characters in the input)
+=item * lexer failed (there are stray characters in the input)
 
-=item unimplemented feature of JSON encountered in input
+=item * unimplemented feature of JSON encountered in input
 
-=item Unicode \uXXXX decoding failed
+=item * Unicode \uXXXX decoding failed
 
-=item input was empty
+=item * input was empty
 
-=item the text did not start with { or [ as it should have
+=item * the text did not start with { or [ as it should have
 
-=item met an unknown escape sequence (backslash \ + character)
-
-=item there was an unparseable number in the input
+=item * met an unknown escape sequence (backslash \ + character)
 
 =back
 
@@ -261,11 +347,7 @@ L<http://www.ietf.org/rfc/rfc4627.txt>.
 
 =back
 
-=head1 HOW IT WORKS
-
-JSON::Parse is based on a parser written in C. The C parser makes use
-of the utility Bison. The C parser is reentrant, in other words
-thread-safe.
+=head1 NOTES
 
 =head2 Performance
 
@@ -278,30 +360,29 @@ L</json_to_perl>, according to the module author's tests is about a
 hundred times faster than L<JSON::PP> and roughly comparable to
 L<JSON::XS>. 
 
-
 =head2 History
 
-This module began as a project to write a JSON parser in C, based on
-callbacks to user-defined routines rather than allocating memory, and
-using the minimum amount of source code. Originally it was written
-with the goal of fitting into 300 lines of C/Flex/Bison source code,
-including comments, and without obfuscation. The original project has
-been modified in order to make the parser reentrant and has grown over
-the intended number of lines of code.
+This module began as a programming exercise to write a JSON parser in
+C, based on callbacks to user-defined routines rather than allocating
+memory, and using the minimum amount of source code. Originally it was
+written with the goal of fitting into 300 lines of C/Flex/Bison source
+code, including comments, and without obfuscation. The original
+project has been modified in order to make the parser reentrant and
+has grown over the intended number of lines of code.
 
-The module was previously known as JSON::Argo. With version 0.05, the
-name was changed to JSON::Parse because, unknown to this module's
-author, there was already a JSON parser called Argo written in Java.
+The Perl module was initially released as JSON::Argo (Argo is the name
+of Jason's ship in Homer's Odyssey). With version 0.05, the name was
+changed to JSON::Parse because, unknown to this module's author, there
+was already a JSON parser called Argo written in Java.
 
 Version 0.06 moved from using a Flex lexer to a C lexer in order to
 increase the speed of the module, to be competitive with L<JSON::XS>.
 
-=head1 EXPORTS
+=head2 Source code
 
-The module exports nothing by default. Functions L</json_to_perl> and
-L</valid_json> can be exported on request.
-
-=head1 SOURCES
+JSON::Parse is based on a parser written in C. The C parser uses a
+utility called Bison. The C parser is reentrant, in other words
+thread-safe.
 
 One of the C files in the distribution is the output of the "bison"
 program. The original input to the "bison" program may be found in the
@@ -310,6 +391,11 @@ installed "bison" to build this module. The bison input is included in
 the distribution to comply with the requirements of the GNU General
 Public License.
 
+=head1 EXPORTS
+
+The module exports nothing by default. Functions L</json_to_perl> and
+L</valid_json> can be exported on request.
+
 =head1 AUTHOR
 
 Ben Bullock, <bkb@cpan.org>
